// Dynamic place contents -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "PlaceContents.h"
#include <cassert>
#include "Place.h"
#include "GlobalMarking.h"
#include "Valuation.h"
#include "Printer.h"

/** @file PlaceContents.C
 * Operation for reading the marking of a place
 */

PlaceContents::PlaceContents (const class Place& place) :
  myPlace (place)
{
  setType (place.getType ());
}

PlaceContents::~PlaceContents ()
{
}

class PlaceMarking*
PlaceContents::meval (const class Valuation& valuation) const
{
  if (const class GlobalMarking* m = valuation.getGlobalMarking ())
    return new class PlaceMarking ((*m)[myPlace.getIndex ()]);
  else {
    valuation.flag (errVar, *this);
    return NULL;
  }
}

bool
PlaceContents::depends (const class VariableSet&,
			bool) const
{
  return false;
}

bool
PlaceContents::forVariables (bool (*operation)
			     (const class Expression&,void*),
			     void* data) const
{
  return operation (*this, data);
}

#ifdef EXPR_COMPILE
# include "CExpression.h"

void
PlaceContents::compileScalarMset (class CExpression& cexpr,
				  unsigned indent,
				  const char* result,
				  const class VariableSet*,
				  bool check) const
{
  class StringBuffer& out = cexpr.getOut ();
  if (check) {
    out.indent (indent);
    out.append ("if (");
    out.append (result);
    if (myPlace.getMaxNumTokens () == 1) {
      out.append (" && ");
      out.append (cexpr.getMultiset ());
      out.append (".p");
      out.append (myPlace.getIndex ());
    }
    else {
      out.append (" && singleton (");
      out.append (cexpr.getMultiset ());
      out.append (".p");
      out.append (myPlace.getIndex ());
      out.append (")");
    }
    out.append (")\n");
    cexpr.compileError (indent + 2, errConst);
  }
  if (myPlace.getMaxNumTokens () == 1) {
    out.indent (indent);
    out.append (result);
    out.append (" = ");
    out.append (cexpr.getMultiset ());
    out.append (".p");
    out.append (myPlace.getIndex ());
    out.append (";\n");
  }
  else {
    out.indent (indent);
    out.append (result);
    out.append (" = singleton (");
    out.append (cexpr.getMultiset ());
    out.append (".p");
    out.append (myPlace.getIndex ());
    out.append (");\n");
  }
}

void
PlaceContents::compileMset (class CExpression& cexpr,
			    unsigned indent,
			    const char* resulttype,
			    const char* result,
			    const class VariableSet*) const
{
  class StringBuffer& out = cexpr.getOut ();
  if (myPlace.getMaxNumTokens () == 1) {
    out.indent (indent);
    if (myPlace.getCapacityBits ()) {
      out.append ("if (");
      out.append (cexpr.getMultiset ());
      out.append (".p");
      out.append (myPlace.getIndex ());
      out.append (")\n");
      out.indent (indent + 2);
    }
    out.append (result);
    out.append ("=insert");
    getType ()->appendIndex (out);
    out.append (" (");
    if (resulttype)
      out.append (resulttype);
    out.append (result);
    out.append (", ");
    out.append (cexpr.getMultiset ());
    out.append (".p");
    out.append (myPlace.getIndex ());
    out.append (", 1);\n");
  }
  else {
    out.indent (indent);
    out.append (result);
    out.append ("=copy");
    getType ()->appendIndex (out);
    out.append (" (");
    if (resulttype)
      out.append (resulttype);
    out.append (result);
    out.append (", ");
    out.append (cexpr.getMultiset ());
    out.append (".p");
    out.append (myPlace.getIndex ());
    out.append (");\n");
  }
}

char*
PlaceContents::getName (const class CExpression& cexpr) const
{
  const char* mset = cexpr.getMultiset ();
  assert (!!mset);
  class StringBuffer buf;
  buf.append (mset);
  buf.append (".p");
  buf.append (myPlace.getIndex ());
  char* name = new char[buf.getLength () + 1];
  memcpy (name, buf.getString (), buf.getLength () + 1);
  return name;
}

#endif // EXPR_COMPILE

void
PlaceContents::display (const class Printer& printer) const
{
  printer.printRaw ("place");
  printer.delimiter (' ');
  printer.print (myPlace.getName ());
}
